@@ -186,6 +186,12 @@ void book_moves(list_t * list, const board_t * board) {
 
    if(BookFile==NULL) return;
 
+   // null keys are reserved for the header
+   if(board->key==U64(0x0)) return;
+   
+
+
+
    // init
 
    list_clear(list);
